#!/usr/bin/python3

#  Copyright 2013 Linaro Limited
#  Author Matt Hart <matthew.hart@linaro.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.

from pdudaemon.drivers.acme import ACME  # pylint: disable=W0611
from pdudaemon.drivers.anelnetpwrctrl import AnelNETPwrCtrlHOME  # pylint: disable=W0611
from pdudaemon.drivers.anelnetpwrctrl import AnelNETPwrCtrlADV  # pylint: disable=W0611
from pdudaemon.drivers.anelnetpwrctrl import AnelNETPwrCtrlIO  # pylint: disable=W0611
from pdudaemon.drivers.anelnetpwrctrl import AnelNETPwrCtrlPRO  # pylint: disable=W0611
from pdudaemon.drivers.apc7900 import APC7900  # pylint: disable=W0611
from pdudaemon.drivers.apc7932 import APC7932  # pylint: disable=W0611
from pdudaemon.drivers.apc7952 import APC7952  # pylint: disable=W0611
from pdudaemon.drivers.apc9218 import APC9218  # pylint: disable=W0611
from pdudaemon.drivers.apc8959 import APC8959  # pylint: disable=W0611
from pdudaemon.drivers.apc9210 import APC9210  # pylint: disable=W0611
from pdudaemon.drivers.apc7920 import APC7920  # pylint: disable=W0611
from pdudaemon.drivers.apc7921 import APC7921  # pylint: disable=W0611
from pdudaemon.drivers.cleware import ClewareUsbSwitch4
from pdudaemon.drivers.ubiquity import Ubiquity3Port  # pylint: disable=W0611
from pdudaemon.drivers.ubiquity import Ubiquity6Port  # pylint: disable=W0611
from pdudaemon.drivers.ubiquity import Ubiquity8Port  # pylint: disable=W0611
from pdudaemon.drivers.localcmdline import LocalCmdline
from pdudaemon.drivers.ip9258 import IP9258
from pdudaemon.drivers.sainsmart import Sainsmart
from pdudaemon.drivers.devantech import DevantechETH002
from pdudaemon.drivers.devantech import DevantechETH0621
from pdudaemon.drivers.devantech import DevantechETH484
from pdudaemon.drivers.devantech import DevantechETH008
from pdudaemon.drivers.devantech import DevantechETH8020
from pdudaemon.drivers.devantech import DevantechDS2824
from pdudaemon.drivers.devantechusb import DevantechUSB2
from pdudaemon.drivers.devantechusb import DevantechUSB8
from pdudaemon.drivers.numatousb import NumatoUSB1
from pdudaemon.drivers.numatousb import NumatoUSB2
from pdudaemon.drivers.numatousb import NumatoUSB4
from pdudaemon.drivers.numatousb import NumatoUSB8
from pdudaemon.drivers.numatousb import NumatoUSB16
from pdudaemon.drivers.numatousb import NumatoUSB32
from pdudaemon.drivers.numatousb import NumatoUSB64
from pdudaemon.drivers.servertechpro2 import ServerTechPro2  # pylint: disable=W0611
from pdudaemon.drivers.synaccess import SynNetBooter
from pdudaemon.drivers.tasmota import SonoffS20Tasmota  # pylint: disable=W0611
from pdudaemon.drivers.tasmota import BrennenstuhlWSPL01Tasmota  # pylint: disable=W0611
from pdudaemon.drivers.egpms import EgPMS
from pdudaemon.drivers.ykush import YkushXS
from pdudaemon.drivers.ykush import Ykush
from pdudaemon.drivers.snmp import SNMP
from pdudaemon.drivers.energenieusb import EnerGenieUSB
from pdudaemon.drivers.bcu import BCU
from pdudaemon.drivers.vusbhid import VUSBHID
from pdudaemon.drivers.tplink import TPLink
from pdudaemon.drivers.ip9850 import ip9850
from pdudaemon.drivers.intellinet import Intellinet
from pdudaemon.drivers.esphome import ESPHomeHTTP
from pdudaemon.drivers.servo import Servo
